@@ -46,6 +46,7 @@
 #endif
 #include "stuff/imsrtns.h"
 #include "stuff/latin1.h"
+#include "stuff/utf-16.h"
 
 #ifdef MDB_DEBUG
 #define DEBUG_PRINT(...) do { fprintf(stderr, __VA_ARGS__); } while (0)
@@ -583,7 +584,13 @@ int mdbInit (const struct configAPI_t *configAPI)
 		return 0;
 	}
 	sprintf (path, "%sCPMODNFO.DAT", CFDATAHOMEDIR);
+#ifdef _WIN32
+	uint16_t *wpath = utf8_to_utf16 (path);
+	fwprintf(stderr, L"Loading %ls .. ", wpath);
+	free (wpath);
+#else
 	fprintf(stderr, "Loading %s .. ", path);
+#endif
 
 	mdbFile = osfile_open_readwrite (path, 1, 0);
 	free (path);
