@@ -101,6 +101,12 @@ public:
 private:
     using sidBankMap_t = std::map<int, ExtraSidBank*>;
 
+public:
+    c64vic getVIC(void)
+    {
+        return vic;
+    }
+
 private:
     /// System clock frequency
     double cpuFrequency;
@@ -150,6 +156,8 @@ private:
 private:
     static double getCpuFreq(model_t model);
 
+    static void deleteSids(sidBankMap_t &extraSidBanks);
+
 private:
     /**
      * IRQ trigger signal.
