@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2022-'24 Stian Sebastian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2022-'26 Stian Sebastian Skjelstad <stian.skjelstad@gmail.com>
  *
  * libsidplay setup config dialog
  *
@@ -20,21 +20,10 @@
 
 #include "config.h"
 #include <assert.h>
-#include <ctype.h>
-#include <dirent.h>
-#include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/stat.h>
 #include <sys/types.h>
-#include <unistd.h>
-#ifdef _WIN32
-# include <windows.h>
-# include <fileapi.h>
-# include <windows.h>
-#endif
 #include "types.h"
 #include "boot/plinkman.h"
 #include "boot/psetting.h"
@@ -47,7 +36,6 @@
 #include "stuff/compat.h"
 #include "stuff/err.h"
 #include "stuff/imsrtns.h"
-#include "stuff/utf-8.h"
 #include "stuff/poutput.h"
 
 #include "sidconfig.h"
@@ -66,6 +54,7 @@ struct browser_t
 	int isparent;
 #ifdef _WIN32
 	int isdrive;
+	char drive;
 #endif
 	uint32_t dirdb_ref;
 	char hash_4096[33];
@@ -84,7 +73,6 @@ static int config_defaultSID;
 static int config_forceSID;
 static int config_CIA;
 static int config_filter;
-static int config_filterbias;
 static int config_filtercurve6581;
 static int config_filterrange6581;
 static int config_filtercurve8580;
@@ -302,18 +290,15 @@ static void ConfigDrawHashMenuInfo (const int lineno, int xpos, int width, const
  |  5: force SID:       [yes] [no]                                            | ignore information in SID files?
  |  6: CIA:             [MOS6526] [MOS6526W4485] [MOS8521]                    | MOS6526 is the classic chip, MOS6526W4485 is a specific batch while MOS8521 is the modern chip
  |  7: filter:          [yes] [no]                                            |
-                                          12345678901 12345678901 = 23
-                                  123456
- |  8: filterbias:         0.0mV  -500.0 [...........#...........] 500.0      | Default is 0.0mV
- |  9: filtercurce6581:    0.50      0.0 [           #           ]   1.0      | Default is 0.5
- | 10: filtercurce8580:    0.50      0.0 [           #           ]   1.0      | Default is 0.5
- | 11: digiboost:       [yes] [no]                                            |
- | 12: kernal.rom:      sadfasdfsdfsadf/sdfasdf/sdfasdf/sdfasdf.ROM           | KERNEL.ROM images can be found online. Some SID files requires this file in order to play correctly
- |                           sadfløjhsdflkhjasdf
- | 13: basic.rom:       sadfasdfsdfsadf/sdfasdf/sdfasdf/sdfasdf.ROM           | BASIC.ROM images can be found online. Some SID files requires this file in order to play correctly
- |                           sadfløjhsdflkhjasdf
- | 14: chargen.rom:     sadfasdfsdfsadf/sdfasdf/sdfasdf/sdfasdf.ROM           | CHARGEN.ROM images can be found online. Some SID files requires this file in order to play correctly
- |                           sadfløjhsdflkhjas
+ |  8: filtercurve6581:    0.50      0.0 [           #           ]   1.0      | Default is 0.5
+ |  9: filtercurve8580:    0.50      0.0 [           #           ]   1.0      | Default is 0.5
+ | 10: digiboost:       [yes] [no]                                            |
+ | 11: kernal.rom:      sadfasdfsdfsadf/sdfasdf/sdfasdf/sdfasdf.ROM           | KERNEL.ROM images can be found online. Some SID files requires this file in order to play correctly
+ |                           sadflojhsdflkhjasdf                              |
+ | 12: basic.rom:       sadfasdfsdfsadf/sdfasdf/sdfasdf/sdfasdf.ROM           | BASIC.ROM images can be found online. Some SID files requires this file in order to play correctly
+ |                           sadflojhsdflkhjasdf                              |
+ | 13: chargen.rom:     sadfasdfsdfsadf/sdfasdf/sdfasdf/sdfasdf.ROM           | CHARGEN.ROM images can be found online. Some SID files requires this file in order to play correctly
+ |                           sadflojhsdflkhjas                                |
  +----------------------------------------------------------------------------|
  |                                                                            |
  |                                                                            |
@@ -325,12 +310,12 @@ static void sidConfigDraw (int EditPos, const struct DevInterfaceAPI_t *API)
 	const int LINES_NOT_AVAILABLE_START = 3;
 	const int LINES_NOT_AVAILABLE_STOP = 4;
 	const int LINES_NOT_AVAILABLE = LINES_NOT_AVAILABLE_START + LINES_NOT_AVAILABLE_STOP;
-	const int maxcontentheight = 19;
+	const int maxcontentheight = 18;
 	      int contentheight = MIN(maxcontentheight, MAX(API->console->TextHeight - 1 - LINES_NOT_AVAILABLE, LINES_NOT_AVAILABLE + 1));
 	      int mlHeight = contentheight + LINES_NOT_AVAILABLE;
 	      int mlTop, mlLeft, mlWidth;
 	const char *offon[] = {"off", "on"};
-	const char *emulators[] = {"resid", "residfp"};
+	const char *emulators[] = {"residfp", "residfpII"};
 	const char *C64models[] = {"PAL", "NTSC", "OLD-NTSC", "DREAN", "PAL-M"};
 	const char *SIDmodels[] = {"MOS6581", "MOS8580"};
 	const char *CIAmodels[] = {"MOS6526", "MOS6526W4485", "MOS8521"};
@@ -389,17 +374,16 @@ if (skip)                   \
 	_A ConfigDrawMenuItems    (mlTop++, mlLeft, mlWidth, " 5: force SID", offon, 2, config_forceSID, EditPos==4, API);                      _B
 	_A ConfigDrawMenuItems    (mlTop++, mlLeft, mlWidth, " 6: CIA", CIAmodels, 3, config_CIA, EditPos==5, API);                             _B
 	_A ConfigDrawMenuItems    (mlTop++, mlLeft, mlWidth, " 7: filter", offon, 2, config_filter, EditPos==6, API);                           _B
-	_A ConfigDrawMenuBar      (mlTop++, mlLeft, mlWidth, " 8: filterbias", 10, "mV", -5000, 5000, config_filterbias, EditPos==7, API);      _B
-	_A ConfigDrawMenuBar      (mlTop++, mlLeft, mlWidth, " 9: filtercurve6581", 100, "", 0, 100, config_filtercurve6581, EditPos==8, API);  _B
-	_A ConfigDrawMenuBar      (mlTop++, mlLeft, mlWidth, "10: filterrange6581", 100, "", 0, 100, config_filterrange6581, EditPos==9, API);  _B
-	_A ConfigDrawMenuBar      (mlTop++, mlLeft, mlWidth, "11: filtercurve8580", 100, "", 0, 100, config_filtercurve8580, EditPos==10, API); _B
-	_A ConfigDrawMenuItems    (mlTop++, mlLeft, mlWidth, "12: CWS", combinedwaveforms, 3, config_combinedwaveforms, EditPos==11, API);      _B
-	_A ConfigDrawMenuItems    (mlTop++, mlLeft, mlWidth, "13: digiboost", offon, 2, config_digiboost, EditPos==12, API);                    _B
-	_A ConfigDrawMenuRom      (mlTop++, mlLeft, mlWidth, "14: kernal.rom:", EditPos==13, config_kernal, API);                               _B
+	_A ConfigDrawMenuBar      (mlTop++, mlLeft, mlWidth, " 8: filtercurve6581", 100, "", 0, 100, config_filtercurve6581, EditPos==7, API);  _B
+	_A ConfigDrawMenuBar      (mlTop++, mlLeft, mlWidth, " 9: filterrange6581", 100, "", 0, 100, config_filterrange6581, EditPos==8, API);  _B
+	_A ConfigDrawMenuBar      (mlTop++, mlLeft, mlWidth, "10: filtercurve8580", 100, "", 0, 100, config_filtercurve8580, EditPos==9, API);  _B
+	_A ConfigDrawMenuItems    (mlTop++, mlLeft, mlWidth, "11: CWS", combinedwaveforms, 3, config_combinedwaveforms, EditPos==10, API);      _B
+	_A ConfigDrawMenuItems    (mlTop++, mlLeft, mlWidth, "12: digiboost", offon, 2, config_digiboost, EditPos==11, API);                    _B
+	_A ConfigDrawMenuRom      (mlTop++, mlLeft, mlWidth, "13: kernal.rom:", EditPos==12, config_kernal, API);                               _B
 	_A ConfigDrawHashMenuInfo (mlTop++, mlLeft, mlWidth, entry_kernal.hash_8192, entry_kernal.hash_4096, 0, API);                           _B
-	_A ConfigDrawMenuRom      (mlTop++, mlLeft, mlWidth, "15: basic.rom:", EditPos==14, config_basic, API);                                 _B
+	_A ConfigDrawMenuRom      (mlTop++, mlLeft, mlWidth, "14: basic.rom:", EditPos==13, config_basic, API);                                 _B
 	_A ConfigDrawHashMenuInfo (mlTop++, mlLeft, mlWidth, entry_basic.hash_8192, entry_basic.hash_4096, 1, API);                             _B
-	_A ConfigDrawMenuRom      (mlTop++, mlLeft, mlWidth, "16: chargen.rom", EditPos==15, config_chargen, API);                              _B
+	_A ConfigDrawMenuRom      (mlTop++, mlLeft, mlWidth, "15: chargen.rom", EditPos==14, config_chargen, API);                              _B
 	_A ConfigDrawHashMenuInfo (mlTop++, mlLeft, mlWidth, entry_chargen.hash_8192, entry_chargen.hash_4096, 2, API);                         _B
 
 #undef _A
@@ -410,8 +394,8 @@ if (skip)                   \
 	switch (EditPos)
 	{
 		case 0:
-			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " resid   = standard integer emulator - fastest.");
-			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " residfp = floating point emulator - better quality but slower.");
+			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " residfp   = floating point emulator.");
+			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " residfpII = floating point emulator, mark II.");
 			break;
 		case 1:
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " For SID files that does not specify C64 model.");
@@ -438,38 +422,34 @@ if (skip)                   \
 			mlTop++;
 			break;
 		case 7:
-			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Only used by \"resid\".");
-			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Default is 0.0mV");
-			break;
-		case 8:
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Only used by \"residfp\". Value to use if SID is MOS6581.");
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Default is 0.5");
 			break;
-		case 9:
+		case 8:
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Only used by \"residfp\". MOS6581 only: 0=\"bright\", 1=\"dark\".");
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Default is 0.5%");
 			break;
-		case 10:
+		case 9:
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Only used by \"residfp\". Value to use if SID is MOS8580.");
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Default is 0.5");
 			break;
-		case 11:
+		case 10:
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Only used by \"residfp\". Combined waveforms strength.");
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Default is average.");
 			break;
-		case 12:
+		case 11:
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " Digi-Boost is a hardware feature only available on MOS8580, where the");
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " digital playback would be boosted.");
 			break;
-		case 13:
+		case 12:
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " KERNEL.ROM images can be found online. Some SID files requires this file");
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " in order to play correctly.");
 			break;
-		case 14:
+		case 13:
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " BASIC.ROM images can be found online. Some SID files requires this file");
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " in order to play correctly.");
 			break;
-		case 15:
+		case 14:
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " CHARGEN.ROM images can be found online. Some SID files requires this file");
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " in order to play correctly.");
 			break;
@@ -482,8 +462,8 @@ if (skip)                   \
 
 static int emulator_to_int (const char *src)
 {
-	if (!strcasecmp (src, "resid")) return 0;
-	if (!strcasecmp (src, "residfp")) return 1;
+	if (!strcasecmp (src, "residfp")) return 0;
+	if (!strcasecmp (src, "residfpII")) return 1;
 	return 1;
 }
 
@@ -491,9 +471,9 @@ static const char *emulator_from_int (const int src)
 {
 	switch (src)
 	{
+		case 0: return "residfp";
 		default:
-		case 0: return "resid";
-		case 1: return "residfp";
+		case 1: return "residfpII";
 	}
 }
 
@@ -575,6 +555,7 @@ static const char *CWS_from_int (const int src)
 	}
 }
 
+#if 0
 static const char *int_to_float10x(int src)
 {
 	static char retval[32];
@@ -582,6 +563,7 @@ static const char *int_to_float10x(int src)
 	snprintf (retval, sizeof (retval), "%d.%01d", src / 10, src_abs % 10);
 	return retval;
 }
+#endif
 
 static const char *int_to_float100x(int src)
 {
@@ -591,6 +573,7 @@ static const char *int_to_float100x(int src)
 	return retval;
 }
 
+#if 0
 static int float10x_to_int(const char *src)
 {
 	int retval = atoi (src) * 10;
@@ -604,6 +587,7 @@ static int float10x_to_int(const char *src)
 	}
 	return retval;
 }
+#endif
 
 static int float100x_to_int(const char *src)
 {
@@ -623,90 +607,54 @@ static int float100x_to_int(const char *src)
 	return retval;
 }
 
-static void rom_md5 (char md5[33], uint32_t dirdb_ref, int size, const struct DevInterfaceAPI_t *API)
+static void rom_md5s (char md5_4096[33], char md5_8192[33], uint32_t dirdb_ref, const struct DevInterfaceAPI_t *API)
 {
-	char *romPath = 0;
-#ifdef _WIN32
-	HANDLE f;
-#else
-	int fd;
-#endif
-	uint8_t buffer[4096];
-	MD5_CTX ctx;
+	struct ocpfile_t *file = 0;
+	struct ocpfilehandle_t *h;
+	uint8_t buffer[8192];
+	int fill = 0;
 
-	md5[0] = '-';
-	md5[1] = 0;
-	md5[32] = 0;
+	md5_4096[0] = '-';
+	md5_4096[1] = 0;
+	md5_4096[32] = 0;
 
-#ifdef _WIN32
-	API->dirdb->GetFullname_malloc (dirdb_ref, &romPath, DIRDB_FULLNAME_DRIVE | DIRDB_FULLNAME_BACKSLASH);
+	md5_8192[0] = '-';
+	md5_8192[1] = 0;
+	md5_8192[32] = 0;
 
-	f = CreateFile (romPath,             /* lpFileName */
-	                GENERIC_READ,          /* dwDesiredAccess */
-	                FILE_SHARE_READ,       /* dwShareMode */
-	                0,                     /* lpSecurityAttributes */
-	                OPEN_EXISTING,         /* dwCreationDisposition */
-	                FILE_ATTRIBUTE_NORMAL, /* dwFlagsAndAttributes */
-	                0);                    /* hTemplateFile */
-	free (romPath);
-	if (f == INVALID_HANDLE_VALUE)
+	API->filesystem_resolve_dirdb_file (dirdb_ref, 0, &file);
+	if (!file)
 	{
 		return;
 	}
-#else
-	API->dirdb->GetFullname_malloc (dirdb_ref, &romPath, DIRDB_FULLNAME_NODRIVE);
-	fd = open (romPath, O_RDONLY);
-	free (romPath);
-	if (fd < 0)
+	h = file->open (file);
+	if (h)
 	{
-		return;
-	}
-#endif
-
-	MD5Init (&ctx);
-
-	while (size)
-	{
-#ifdef _WIN32
-		DWORD NumberOfBytesRead = 0;
-		BOOL Result;
-		Result = ReadFile (f, buffer, 4096, &NumberOfBytesRead, 0);
-		if ((!Result) || (NumberOfBytesRead != 4096))
-		{
-			CloseHandle (f);
-			return;
-		}
-		MD5Update (&ctx, buffer, 4096);
-		size -= 4096;
-#else
-		int result = read (fd, buffer, 4096);
-		if (result < 0)
-		{
-			if (errno == EINTR)
-			{
-				continue;
-			}
-			close (fd);
-			return;
-		}
-		if (result == 0)
-		{
-			close (fd);
-			return;
-		}
-		MD5Update (&ctx, buffer, 4096);
-		size -= result;
-#endif
+		fill = h->read (h, buffer, 8192);
+		h->unref (h);
+		h = 0;
 	}
 
-#ifdef _WIN32
-	CloseHandle (f);
-#else
-	close (fd);
-#endif
-	MD5Final (md5, &ctx);
+	file->unref (file);
+	file = 0;
+
+	if (fill >= 4096)
+	{
+		MD5_CTX ctx;
+		MD5Init (&ctx);
+		MD5Update (&ctx, buffer, 4096);
+		MD5Final (md5_4096, &ctx);
+	}
+	if (fill >= 8192)
+	{
+		MD5_CTX ctx;
+		MD5Init (&ctx);
+		MD5Update (&ctx, buffer, 8192);
+		MD5Final (md5_8192, &ctx);
+	}
 }
 
+
 static void sidDrawDir (const int esel, const int expect, const struct DevInterfaceAPI_t *API)
 {
 	int contentheight = entries_count;
@@ -791,12 +739,17 @@ static void sidDrawDir (const int esel, const int expect, const struct DevInterf
 		{
 			if (entries_data[masterindex].isparent)
 			{
-				API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, "%*.*o..%0.7o ", (masterindex==esel)?0x6:0x0, (masterindex==esel)?0x1:0x1);
+				API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, "%*.*o..", (masterindex==esel)?0x6:0x0, (masterindex==esel)?0x1:0x1);
 			} else {
 				const char *dirname;
 				API->dirdb->GetName_internalstr (entries_data[masterindex].dirdb_ref, &dirname);
-				API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, "%*.*o%*S%0.7o ", (masterindex==esel)?0x6:0x0, (masterindex==esel)?0x1:0x1, mlWidth - 2, dirname);
+				API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, "%*.*o%*S", (masterindex==esel)?0x6:0x0, (masterindex==esel)?0x1:0x1, mlWidth, dirname);
 			}
+#ifdef _WIN32
+		} else if (entries_data[masterindex].isdrive)
+		{
+				API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, "%*.*o%c:", (masterindex==esel)?0x6:0x0, (masterindex==esel)?0x1:0x1, entries_data[masterindex].drive);
+#endif
 		} else {
 			const char *filename;
 			API->dirdb->GetName_internalstr (entries_data[masterindex].dirdb_ref, &filename);
@@ -855,6 +808,10 @@ static int cmp(const void *a, const void *b)
 	{
 		return 1;
 	}
+	if (p1->isdrive)
+	{
+		return p1->drive - p2->drive;
+	}
 #endif
 
 	cmp_API->dirdb->GetName_internalstr (p1->dirdb_ref, &n1);
@@ -890,7 +847,6 @@ static void entries_append_parent (uint32_t ref, const struct DevInterfaceAPI_t
 {
 	if (entries_append())
 	{
-		API->dirdb->Unref (ref, dirdb_use_file);
 		return;
 	}
 	entries_data[entries_count].isdir = 1;
@@ -898,6 +854,7 @@ static void entries_append_parent (uint32_t ref, const struct DevInterfaceAPI_t
 #ifdef _WIN32
 	entries_data[entries_count].isdrive = 0;
 #endif
+	API->dirdb->Ref (ref, dirdb_use_file);
 	entries_data[entries_count].dirdb_ref = ref;
 	entries_data[entries_count].hash_4096[0] = 0;
 	entries_data[entries_count].hash_8192[0] = 0;
@@ -908,7 +865,6 @@ static void entries_append_dir (uint32_t ref, const struct DevInterfaceAPI_t *AP
 {
 	if (entries_append())
 	{
-		API->dirdb->Unref (ref, dirdb_use_file);
 		return;
 	}
 	entries_data[entries_count].isdir = 1;
@@ -916,6 +872,7 @@ static void entries_append_dir (uint32_t ref, const struct DevInterfaceAPI_t *AP
 #ifdef _WIN32
 	entries_data[entries_count].isdrive = 0;
 #endif
+	API->dirdb->Ref (ref, dirdb_use_file);
 	entries_data[entries_count].dirdb_ref = ref;
 	entries_data[entries_count].hash_4096[0] = 0;
 	entries_data[entries_count].hash_8192[0] = 0;
@@ -926,11 +883,14 @@ static void entries_append_file (uint32_t ref, const char *hash_4096, const char
 {
 	if (entries_append())
 	{
-		API->dirdb->Unref (ref, dirdb_use_file);
 		return;
 	}
 	entries_data[entries_count].isdir = 0;
 	entries_data[entries_count].isparent = 0;
+#ifdef _WIN32
+	entries_data[entries_count].isdrive = 0;
+#endif
+	API->dirdb->Ref (ref, dirdb_use_file);
 	entries_data[entries_count].dirdb_ref = ref;
 	strcpy (entries_data[entries_count].hash_4096, hash_4096);
 	strcpy (entries_data[entries_count].hash_8192, hash_8192);
@@ -938,7 +898,7 @@ static void entries_append_file (uint32_t ref, const char *hash_4096, const char
 }
 
 #ifdef _WIN32
-static void entries_append_drive (uint32_t ref, const struct DevInterfaceAPI_t *API)
+static void entries_append_drive (uint32_t ref, const struct DevInterfaceAPI_t *API, char drive)
 {
 	if (ref == DIRDB_CLEAR)
 	{
@@ -951,7 +911,8 @@ static void entries_append_drive (uint32_t ref, const struct DevInterfaceAPI_t *
 	entries_data[entries_count].isdir = 0;
 	entries_data[entries_count].isparent = 0;
 	entries_data[entries_count].isdrive = 1;
-	API->dirdb->Unref(ref, dirdb_use_file);
+	entries_data[entries_count].drive = drive;
+	API->dirdb->Ref(ref, dirdb_use_file);
 	entries_data[entries_count].dirdb_ref = ref;
 	entries_data[entries_count].hash_4096[0] = 0;
 	entries_data[entries_count].hash_8192[0] = 0;
@@ -959,9 +920,51 @@ static void entries_append_drive (uint32_t ref, const struct DevInterfaceAPI_t *
 }
 #endif
 
+static void refresh_dir_add_file (void *token, struct ocpfile_t *file)
+{
+	const struct DevInterfaceAPI_t *API = token;
+	const char *filename = 0;
+	size_t len;
+	API->dirdb->GetName_internalstr (file->dirdb_ref, &filename);
+	if (!filename)
+	{
+		return;
+	}
+	len = strlen (filename);
+	if (len <= 4)
+	{
+		return;
+	}
+	if ((!strcasecmp (filename + len - 4, ".rom")) ||
+	    (!strcasecmp (filename + len - 4, ".bin")))
+	{
+		char md5_4096[33];
+		char md5_8192[33];
+		rom_md5s (md5_4096, md5_8192, file->dirdb_ref, API);
+		entries_append_file (file->dirdb_ref, md5_4096, md5_8192, API);
+	}
+}
+
+static void refresh_dir_add_dir (void *token, struct ocpdir_t *dir)
+{
+	const struct DevInterfaceAPI_t *API = token;
+	const char *dirname = 0;
+	API->dirdb->GetName_internalstr (dir->dirdb_ref, &dirname);
+	if (!dirname)
+	{
+		return;
+	}
+	if ((strcmp (dirname, ".")) &&
+	    (strcmp (dirname, "..")))
+	{
+		entries_append_dir (dir->dirdb_ref, API);
+	}
+}
+
 static void refresh_dir (uint32_t ref, uint32_t old, int *esel, const struct DevInterfaceAPI_t *API)
 {
-	DIR *d;
+	struct ocpdir_t *dir = 0;
+	//DIR *d;
 	int i;
 
 	*esel = 0;
@@ -974,77 +977,36 @@ static void refresh_dir (uint32_t ref, uint32_t old, int *esel, const struct Dev
 	API->dirdb->GetFullname_malloc (ref, &entries_location, DIRDB_FULLNAME_NODRIVE);
 #endif
 
+	API->filesystem_resolve_dirdb_dir (ref, 0, &dir);
+	if (dir)
 	{
+		ocpdirhandle_pt iter;
+		if (dir->parent)
+		{
+			entries_append_parent (dir->parent->dirdb_ref, API);
+		}
+		iter = dir->readdir_start (dir, refresh_dir_add_file, refresh_dir_add_dir, (void *)API);
+		while (dir->readdir_iterate (iter))
+		{
+		}
+		dir->readdir_cancel (iter);
+		dir->unref (dir);
+		dir = 0;
+	} else {
 		uint32_t dir_ref = API->dirdb->GetParentAndRef (ref, dirdb_use_file);
 		if (dir_ref != UINT32_MAX)
 		{
 			entries_append_parent (dir_ref, API);
 		}
+		 API->dirdb->Unref (dir_ref, dirdb_use_file);
 	}
 
-	d = opendir (entries_location);
-	if (d)
-	{
-		struct dirent *de;
-		while ((de = readdir (d)))
-		{
-			struct stat st;
-			if (!strcmp (de->d_name, "."))
-			{
-				continue;
-			}
-			if (!strcmp (de->d_name, ".."))
-			{
-				continue;
-			}
-			{
-				char *temp = malloc (strlen (entries_location) + 1 + strlen (de->d_name) + 1);
-				int res;
-				if (!temp)
-				{
-					continue;
-				}
-				sprintf (temp, "%s/%s", entries_location, de->d_name);
-				res = stat (temp, &st);
-				free (temp);
-				if (res < 0)
-				{
-					continue;
-				}
-			}
-			if ((st.st_mode & S_IFMT) == S_IFDIR)
-			{
-				entries_append_dir (API->dirdb->FindAndRef (ref, de->d_name, dirdb_use_file), API);
-				continue;
-			} else if ((st.st_mode & S_IFMT) == S_IFREG)
-			{
-				int len = strlen (de->d_name);
-				char md5_4096[33];
-				char md5_8192[33];
-				uint32_t newref;
-				if (len < 4)
-				{
-					continue;
-				}
-				if (strcasecmp (de->d_name + len - 4, ".rom") &&
-				    strcasecmp (de->d_name + len - 4, ".bin"))
-				{
-					continue;
-				}
-				newref = API->dirdb->FindAndRef (ref, de->d_name, dirdb_use_file);
-				rom_md5 (md5_4096, newref, 4096, API);
-				rom_md5 (md5_8192, newref, 8192, API);
-				entries_append_file (newref, md5_4096, md5_8192, API);
-			}
-		}
-		closedir (d);
-	}
 #ifdef _WIN32
 	for (i=0; i < 26; i++)
 	{
 		if (API->dmDriveLetters[i])
 		{
-			entries_append_drive (API->dmDriveLetters[i]->cwd->dirdb_ref, API);
+			entries_append_drive (API->dmDriveLetters[i]->cwd->dirdb_ref, API, i + 'A');
 		}
 	}
 #endif
@@ -1071,7 +1033,6 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 	config_forceSID =                          API->configAPI->GetProfileBool   ("libsidplayfp", "forceSID",        0, 0);
 	config_CIA = CIA_to_int                   (API->configAPI->GetProfileString ("libsidplayfp", "CIA",             "MOS6526"));
 	config_filter =                            API->configAPI->GetProfileBool   ("libsidplayfp", "filter",          1, 1);
-	config_filterbias = float10x_to_int       (API->configAPI->GetProfileString ("libsidplayfp", "filterbias",      "0.0"));
 	config_filtercurve6581 = float100x_to_int (API->configAPI->GetProfileString ("libsidplayfp", "filtercurve6581", "0.5"));
 	config_filterrange6581 = float100x_to_int (API->configAPI->GetProfileString ("libsidplayfp", "filterrange6581", "0.5"));
 	config_filtercurve8580 = float100x_to_int (API->configAPI->GetProfileString ("libsidplayfp", "filtercurve8580", "0.5"));
@@ -1081,8 +1042,6 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 	config_basic = strdup                     (API->configAPI->GetProfileString ("libsidplayfp", "basic",           "BASIC.ROM"));
 	config_chargen = strdup                   (API->configAPI->GetProfileString ("libsidplayfp", "chargen",         "CHARGEN.ROM"));
 
-	if (config_filterbias < -5000) config_filterbias = -5000;
-	if (config_filterbias > 5000) config_filterbias = 5000;
 	if (config_filtercurve6581 < 0) config_filtercurve6581 = 0;
 	if (config_filtercurve6581 > 100) config_filtercurve6581 = 100;
 	if (config_filterrange6581 < 0) config_filterrange6581 = 0;
@@ -1094,12 +1053,9 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 	entry_basic.dirdb_ref   = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_basic,   DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
 	entry_chargen.dirdb_ref = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_chargen, DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
 
-	rom_md5 (entry_kernal .hash_8192, entry_kernal .dirdb_ref, 8192, API);
-	rom_md5 (entry_kernal .hash_4096, entry_kernal .dirdb_ref, 4096, API);
-	rom_md5 (entry_basic  .hash_8192, entry_basic  .dirdb_ref, 8192, API);
-	rom_md5 (entry_basic  .hash_4096, entry_basic  .dirdb_ref, 4096, API);
-	rom_md5 (entry_chargen.hash_8192, entry_chargen.dirdb_ref, 8192, API);
-	rom_md5 (entry_chargen.hash_4096, entry_chargen.dirdb_ref, 4096, API);
+	rom_md5s (entry_kernal .hash_4096, entry_kernal .hash_8192, entry_kernal .dirdb_ref, API);
+	rom_md5s (entry_basic  .hash_4096, entry_basic  .hash_8192, entry_basic  .dirdb_ref, API);
+	rom_md5s (entry_chargen.hash_4096, entry_chargen.hash_8192, entry_chargen.dirdb_ref, API);
 
 	while (1)
 	{
@@ -1122,13 +1078,15 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 				{
 					repeat = 1;
 				} else {
+					/*
 					if (esel == 7)
 					{
 						if (repeat < 20)
 						{
 							repeat += 1;
 						}
-					} else {
+					} else
+					*/ {
 						if (repeat < 5)
 						{
 							repeat += 1;
@@ -1142,17 +1100,17 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 			switch (key)
 			{
 				case _KEY_ENTER:
-					if ((esel == 13) || (esel == 14) || (esel == 15))
+					if ((esel == 12) || (esel == 13) || (esel == 14))
 					{
 						uint32_t dir_ref;
 						int dsel = 0;
 						int inner = 1;
 
-						if (esel == 13)
+						if (esel == 12)
 						{
 							dir_ref = API->dirdb->GetParentAndRef (entry_kernal.dirdb_ref, dirdb_use_dir);
 							refresh_dir (dir_ref, entry_kernal.dirdb_ref, &dsel, API);
-						} else if (esel == 14)
+						} else if (esel == 13)
 						{
 							dir_ref = API->dirdb->GetParentAndRef (entry_basic.dirdb_ref, dirdb_use_dir);
 							refresh_dir (dir_ref, entry_basic.dirdb_ref, &dsel, API);
@@ -1191,7 +1149,7 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 						while (inner)
 						{
 							API->fsDraw();
-							sidDrawDir (dsel, esel - 13, API);
+							sidDrawDir (dsel, esel - 12, API);
 							while (inner && API->console->KeyboardHit())
 							{
 								int key = API->console->KeyboardGetChar();
@@ -1223,9 +1181,9 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 										)
 										{
 											uint32_t dir = entries_data[dsel].dirdb_ref;
-											API->dirdb->Ref (dir, dirdb_use_file);
+											API->dirdb->Ref (dir, dirdb_use_dir);
 											refresh_dir (dir, dir_ref, &dsel, API);
-											API->dirdb->Unref (dir_ref, dirdb_use_file);
+											API->dirdb->Unref (dir_ref, dirdb_use_dir);
 											dir_ref = dir;
 										} else {
 											char *newpath = 0;
@@ -1257,21 +1215,21 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 											}
 											if (newpath)
 											{
-												if (esel == 13)
+												if (esel == 12)
 												{
 													API->dirdb->Unref (entry_kernal.dirdb_ref, dirdb_use_file);
 													entry_kernal = entries_data[dsel];
 													API->dirdb->Ref (entry_kernal.dirdb_ref, dirdb_use_file);
 													free (config_kernal);
 													config_kernal = newpath;
-												} else if (esel == 14)
+												} else if (esel == 13)
 												{
 													API->dirdb->Unref (entry_basic.dirdb_ref, dirdb_use_file);
 													entry_basic = entries_data[dsel];
 													API->dirdb->Ref (entry_basic.dirdb_ref, dirdb_use_file);
 													free (config_basic);
 													config_basic = newpath;
-												} else { /* esel == 15 */
+												} else { /* esel == 14 */
 													API->dirdb->Unref (entry_chargen.dirdb_ref, dirdb_use_file);
 													entry_chargen = entries_data[dsel];
 													API->dirdb->Ref (entry_chargen.dirdb_ref, dirdb_use_file);
@@ -1315,7 +1273,6 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 				case 'B':
 				case 'C':
 				case 'D':
-				case 'E':
 					esel = key - 'A' + 9;
 					break;
 
@@ -1330,29 +1287,25 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 						case 5: config_CIA -= (!!config_CIA); break;
 						case 6: config_filter = 0; break;
 						case 7:
-							config_filterbias -= repeat;
-							if (config_filterbias < -5000) config_filterbias = -5000;
-							break;
-						case 8:
 							config_filtercurve6581 -= repeat;
 							if (config_filtercurve6581 < 0) config_filtercurve6581 = 0;
 							break;
-						case 9:
+						case 8:
 							config_filterrange6581 -= repeat;
 							if (config_filterrange6581 < 0) config_filterrange6581 = 0;
 							break;
-						case 10:
+						case 9:
 							config_filtercurve8580 -= repeat;
 							if (config_filtercurve8580 < 0) config_filtercurve8580 = 0;
 							break;
-						case 11:
+						case 10:
 							config_combinedwaveforms -= 1;
 							if (config_combinedwaveforms < 0) config_combinedwaveforms = 0;
 							break;
-						case 12: config_digiboost = 0; break;
+						case 11: config_digiboost = 0; break;
+						case 12:
 						case 13:
 						case 14:
-						case 15:
 							break;
 					}
 					break;
@@ -1367,34 +1320,30 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 						case 5: config_CIA += (config_CIA != 2); break;
 						case 6: config_filter = 1; break;
 						case 7:
-							config_filterbias += repeat;
-							if (config_filterbias > 5000) config_filterbias = 5000;
-							break;
-						case 8:
 							config_filtercurve6581 += repeat;
 							if (config_filtercurve6581 > 100) config_filtercurve6581 = 100;
 							break;
-						case 9:
+						case 8:
 							config_filterrange6581 += repeat;
 							if (config_filterrange6581 > 100) config_filterrange6581 = 100;
 							break;
-						case 10:
+						case 9:
 							config_filtercurve8580 += repeat;
 							if (config_filtercurve8580 > 100) config_filtercurve8580 = 100;
 							break;
-						case 11:
+						case 10:
 							config_combinedwaveforms += 1;
 							if (config_combinedwaveforms > 2) config_combinedwaveforms = 2;
 							break;
-						case 12: config_digiboost = 1; break;
+						case 11: config_digiboost = 1; break;
+						case 12:
 						case 13:
 						case 14:
-						case 15:
 							break;
 					}
 					break;
 				case KEY_DOWN:
-					if (esel < 15)
+					if (esel < 14)
 					{
 						esel++;
 					}
@@ -1424,7 +1373,6 @@ superexit:
 	API->configAPI->SetProfileBool   ("libsidplayfp", "forceSID", config_forceSID);
 	API->configAPI->SetProfileString ("libsidplayfp", "CIA", CIA_from_int (config_CIA));
 	API->configAPI->SetProfileBool   ("libsidplayfp", "filter", config_filter);
-	API->configAPI->SetProfileString ("libsidplayfp", "filterbias", int_to_float10x(config_filterbias));
 	API->configAPI->SetProfileString ("libsidplayfp", "filtercurve6581", int_to_float100x(config_filtercurve6581));
 	API->configAPI->SetProfileString ("libsidplayfp", "filterrange6581", int_to_float100x(config_filterrange6581));
 	API->configAPI->SetProfileString ("libsidplayfp", "filtercurve8580", int_to_float100x(config_filtercurve8580));
