@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * XMPlay .XM module loader
  *
@@ -310,39 +310,42 @@ OCP_INTERNAL int xmpLoadModule (struct cpifaceSessionAPI_t *cpifaceSession, stru
 		} ins2;
 		uint16_t volfade;
 		int k, j;
+		uint32_t res;
 
-		if (file->read (file, &ins1, sizeof (ins1.size)) != sizeof (ins1.size))
+		if ((res = file->read (file, &ins1, sizeof (ins1.size))) != sizeof (ins1.size))
 		{
 			/*
 			cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] read failed #5.1 (%d/%d)\n", i, m->ninst);
 			FreeResources (&r, head2.ninst);
 			return errFormStruc;
 			*/
-			cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] warning, read failed #5.1\n");
+			cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] warning, read failed #5.1: instrument %d/%d size field only read % " PRId32 " of %" PRId32" bytes\n", i + 1, m->ninst, res, sizeof (ins1.size));
 			ins1.size=0;
-		}
-		ins1.size = uint32_little (ins1.size);
-		/* this next block is a fucking hack to support SMALL files */
-		if (ins1.size<4)
-		{
-			cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] Warning, ins1.size<4\n");
 			memset(&ins1, 0, sizeof(ins1));
-		} else if (ins1.size < sizeof(ins1))
-		{
-			cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] Warning, ins1.size<=sizeof(ins1), reading %d more bytes and zeroing last %d bytes\n", (int)(ins1.size-(int)sizeof(ins1.size)), (int)(sizeof(ins1)-(ins1.size-sizeof(ins1.size))));
-			if (file->read (file, ins1.name, ins1.size - sizeof (ins1.size)) != (ins1.size - sizeof (ins1.size)))
-			{
-				cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] read failed #5.2\n");
-				FreeResources (&r, head2.ninst);
-				return errFileRead;
-			}
-			memset(ins1.name + ins1.size-sizeof(ins1.size), 0, sizeof(ins1)-(ins1.size-sizeof(ins1.size)));
 		} else {
-			if (file->read (file, ins1.name, sizeof (ins1) - sizeof (ins1.size)) != (sizeof (ins1) - sizeof (ins1.size)))
+			ins1.size = uint32_little (ins1.size);
+			/* this next block is a fucking hack to support SMALL files */
+			if (ins1.size<4)
 			{
-				cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] read failed #5.2\n");
-				FreeResources (&r, head2.ninst);
-				return errFormStruc;
+				cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] Warning, ins1.size<4\n");
+				memset(&ins1, 0, sizeof(ins1));
+			} else if (ins1.size < sizeof(ins1))
+			{
+				cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] Warning, ins1.size<=sizeof(ins1), reading %d more bytes and zeroing last %d bytes\n", (int)(ins1.size-(int)sizeof(ins1.size)), (int)(sizeof(ins1)-(ins1.size-sizeof(ins1.size))));
+				if (file->read (file, ins1.name, ins1.size - sizeof (ins1.size)) != (ins1.size - sizeof (ins1.size)))
+				{
+					cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] read failed #5.2\n");
+					FreeResources (&r, head2.ninst);
+					return errFileRead;
+				}
+				memset(ins1.name + ins1.size-sizeof(ins1.size), 0, sizeof(ins1)-(ins1.size-sizeof(ins1.size)));
+			} else {
+				if (file->read (file, ins1.name, sizeof (ins1) - sizeof (ins1.size)) != (sizeof (ins1) - sizeof (ins1.size)))
+				{
+					cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] read failed #5.2\n");
+					FreeResources (&r, head2.ninst);
+					return errFormStruc;
+				}
 			}
 		}
 
@@ -630,15 +633,15 @@ bail2:
 			sip->loopstart=samp.loopstart;
 			sip->loopend=samp.loopstart+samp.looplen;
 			sip->samprate=8363;
-			sip->type=mcpSampDelta| ((samp.type&32)?mcpSampStereo:0) | ((samp.type&16)?mcpSamp16Bit:0) | ((samp.type&3)?(((samp.type&3)>=2)?(mcpSampLoop|mcpSampBiDi):mcpSampLoop):0);
-			sip->length=samp.samplen >> (!!(sip->type & mcpSampStereo));
+			sip->type=mcpSampDelta| ((samp.type&32)?mcpSampInterleavedStereo:0) | ((samp.type&16)?mcpSamp16Bit:0) | ((samp.type&3)?(((samp.type&3)>=2)?(mcpSampLoop|mcpSampBiDi):mcpSampLoop):0);
+			sip->length=samp.samplen >> (!!(sip->type & mcpSampInterleavedStereo));
 		}
 
 		for (j=0; j<ins1.samp; j++)
 		{
 			struct xmpsample *sp=&r.msmps[i][j];
 			struct sampleinfo *sip=&r.smps[i][j];
-			uint32_t l=sip->length<<((!!(sip->type&mcpSamp16Bit)) + (!!(sip->type & mcpSampStereo)));
+			uint32_t l=sip->length<<((!!(sip->type&mcpSamp16Bit)) + (!!(sip->type & mcpSampInterleavedStereo)));
 			if (!l)
 				continue;
 			sip->ptr=malloc(sizeof(uint8_t)*(l+528));
@@ -648,11 +651,14 @@ bail2:
 				FreeResources (&r, head2.ninst);
 				return errAllocMem;
 			}
-			if (file->read (file, sip->ptr, l) != l)
+			if ((res = file->read (file, sip->ptr, l)) != l)
 			{
-				cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] read failed #8\n");
+				cpifaceSession->cpiDebug (cpifaceSession, "[XM/XM] warning, read failed #8: instrument %d/%d, sample %d/%d only read %" PRId32 " of %" PRId32" bytes\n", i + 1, m->ninst, j + 1, ins1.samp, res, l);
+				/*
 				FreeResources (&r, head2.ninst);
 				return errFileRead;
+				*/
+				memset (sip->ptr + res, 0, l - res);
 			}
 			sp->handle=m->nsampi++;
 		}
